#
# spec file for package google-or-tools
#
# Copyright (c) 2023 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define sonum 9
%bcond_without tests

Name:           google-or-tools
Version:        9.6
Release:        3.4
License:        Apache-2.0
Summary:        Suite for solving combinatorial optimization problems
URL:            https://developers.google.com/optimization/
Group:          Development/Languages/C++
Source:         https://github.com/google/or-tools/archive/refs/tags/v%{version}.tar.gz#/google-or-tools-%{version}.tar.gz
# PATCH-FIX-UPSTREAM - remove bad entries from RUNPATHs
Patch5:         0001-Only-add-relevant-directories-to-sample-RUNPATHs.patch
Patch6:         0002-Only-add-relevant-directories-to-flatzinc-library-ex.patch
Patch7:         0003-Only-add-relevant-directories-to-sat_solver-RUNPATHs.patch
Patch8:         0001-base-Remove-AbslStringify.patch
Patch9:         0001-deps-bump-re2-to-2023-07-01.patch
BuildRequires:  abseil-cpp-devel >= 20230105.0
BuildRequires:  cmake >= 3.18
BuildRequires:  fdupes
BuildRequires:  gcc-c++ >= 7
BuildRequires:  python3-absl-py
BuildRequires:  python3-devel
BuildRequires:  python3-matplotlib
BuildRequires:  python3-mypy-protobuf
BuildRequires:  python3-numpy >= 1.13.1
BuildRequires:  python3-pandas
BuildRequires:  python3-pybind11-devel
BuildRequires:  python3-pytest
BuildRequires:  python3-scipy >= 1.10.0
BuildRequires:  python3-setuptools
BuildRequires:  python3-virtualenv
BuildRequires:  python3-wheel
BuildRequires:  swig >= 4.1.1
BuildRequires:  pkgconfig(eigen3)
BuildRequires:  pkgconfig(protobuf) >= 3.21.12
BuildRequires:  pkgconfig(re2)
BuildRequires:  pkgconfig(zlib)

%description
OR-Tools is an open source software suite for optimization, tuned for
tackling the world's toughest problems in vehicle routing, flows,
integer and linear programming, and constraint programming.

%package -n python3-ortools
Summary:        Suite for solving combinatorial optimization problems
Group:          Development/Languages/Python
Requires:       python3-absl-py
Requires:       python3-numpy

%description -n python3-ortools
Python3 bindings for the OR-Tools suite, for solving combinatorial
optimization problems.

%package minizinc
Summary:        OR-Tools minizinc solver
Group:          Productivity/Scientific/Math
Requires:       minizinc

%description minizinc
Minizinc solver using the OR-Tools suite.

%package -n libortools%{sonum}
Summary:        Suite for solving combinatorial optimization problems
Group:          System/Libraries

%description -n libortools%{sonum}
OR-Tools is an open source software suite for optimization.

%package -n libortools_flatzinc%{sonum}
Summary:        Suite for solving combinatorial optimization problems
Group:          System/Libraries

%description -n libortools_flatzinc%{sonum}
OR-Tools is an open source software suite for optimization.

%package devel
Summary:        Suite for solving combinatorial optimization problems
Group:          Development/Languages/C++
Requires:       libortools%{sonum} = %{version}
Requires:       libortools_flatzinc%{sonum} = %{version}

%description devel
Development files (C/C++) for the OR-Tools suite.

%prep
%autosetup -p1 -n or-tools-%{version}
# Allow doc install
sed -i -e '/CMAKE_DEPENDENT_OPTION(INSTALL_DOC/ s/BUILD_CXX AND BUILD_DOC/BUILD_CXX/' CMakeLists.txt

%build
export PIP_NO_INDEX=1
%global optflags %{optflags} -Wno-error=return-type
%cmake \
  -DCMAKE_INSTALL_DOCDIR=%{_docdir}/%{name}/ \
  -DINSTALL_DOC:BOOL=ON \
  -DBUILD_DEPS:BOOL=OFF \
  -DBUILD_pybind11:BOOL=OFF \
  -DBUILD_PYTHON:BOOL=ON \
  -DVENV_USE_SYSTEM_SITE_PACKAGES:BOOL=ON \
  -DUSE_SCIP:BOOL=OFF \
  -DUSE_COINOR:BOOL=OFF \
  %{nil}
%cmake_build

%install
%cmake_install
rm %{buildroot}/%{_docdir}/%{name}/LICENSE
# Remove duplicate library
rm %{buildroot}/%{python3_sitearch}/ortools/.libs/libortools.so.9
rmdir %{buildroot}/%{python3_sitearch}/ortools/.libs

%post -n libortools%{sonum} -p /sbin/ldconfig
%postun -n libortools%{sonum} -p /sbin/ldconfig
%post -n libortools_flatzinc%{sonum} -p /sbin/ldconfig
%postun -n libortools_flatzinc%{sonum} -p /sbin/ldconfig

%check
%if %{with tests}
# Some tests using e.g. SCIP are not skipped, exclude
# https://github.com/google/or-tools/issues/3261
%define known_fail 'python_linear_solver_model_builder_test'
%if %{__isa_bits} == 32
%ctest --exclude-regex %{known_fail} || true
%else
echo "Run tests known to fail"
%ctest --include-regex %{known_fail} || true
echo "Run good tests"
%ctest --exclude-regex %{known_fail}
%endif
%endif

%files
%doc CONTRIBUTING.md README.md
%license LICENSE
%{_bindir}/*
%{_docdir}/%{name}/{sat,constraint_solver}
%exclude %{_bindir}/fzn-ortools

%files -n python3-ortools
%{python3_sitearch}/ortools
%{python3_sitearch}/ortools-%{version}*info

%files minizinc
%{_bindir}/fzn-ortools
%{_datadir}/minizinc

%files -n libortools%{sonum}
%{_libdir}/libor*.so.%{sonum}*
%exclude %{_libdir}/libortools_flatzinc*.so.%{sonum}*

%files -n libortools_flatzinc%{sonum}
%{_libdir}/libortools_flatzinc*.so.%{sonum}*

%files devel
%{_libdir}/lib*.so
%{_libdir}/cmake/ortools
%{_includedir}/ortools
%{_includedir}/ortools_export.h

%changelog
* Tue Aug  8 2023 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Fix build with re2 2023-07-01, add
  0001-deps-bump-re2-to-2023-07-01.patch
- Cleanup skipped tests
* Sun Jul  9 2023 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Fix build with latest protobuf, add
  0001-base-Remove-AbslStringify.patch
* Thu May 11 2023 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Update to version 9.6:
  * Dependencies
    + SCIP v801 -> v803 (note: now uses an OSI compatible license)
    + abseil 20220623.1 -> 20230105.0
    + Protobuf v21.5 -> v21.12
    + SWIG 4.1.1
  * PDLP
    + Add python wrapper.
  * CP-SAT
    + Performance improvements.
  * GLOP
    + Tweak presolve.
  * ModelBuilder
    + Python: Improve numpy support.
  * Bazel
    + Add pybind11 support.
    + Add java wrapper support.
  * Routing
    + Few improvements (local search)
  * Known Issues:
    + CP-SAT: Ignoring pseudo_costs subsolver returns Invalid
    parameters. (#3706)
- Update to version 9.5:
  * CP-SAT
    + Presolve improvements: max(array), Boolean constraints,
    linear constraints.
    + Interleaved search should be deterministic in parallel.
    + Linear cuts: cleanup square and int_prod cuts; rewrite
    cut pipeline.
    + Fingerprint input model and solution (visible in the log).
    + Scheduling improvements
    + The usual bunch of bugfixes (crash during presolve, crash in
    cuts, infeasible solutions, model infeasible in LNS).
  * GLOP
    + Speedup by rewriting the linear algebra, as well as the
    pivot selection rule
  * Linear Solver
    + add knapsack_interface.cc.
    + Move model_builder API under the linear_solver directory
    (headers and samples).
    + Add support for Gurobi 10.
  * Routing
    + Liberate few parser for various routing challenge.
- Drop obsolete patches:
  * do_not_call_pdlp_solve_test.patch
  * 0003-Reuse-common-add_cxx_example-function-for-Sat-Runner.patch
- Update patches:
  * 0001-Only-add-relevant-directories-to-sample-RUNPATHs.patch
  * 0002-Only-add-relevant-directories-to-flatzinc-library-ex.patch
  * 0003-Only-add-relevant-directories-to-sat_solver-RUNPATHs.patch
- Use PIP_NO_INDEX instead of
  0001-Do-not-try-to-download-ortools-wheel.patch
* Fri Nov 25 2022 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Fix build with SWIG 4.1
* Sat Sep 17 2022 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Update to version 9.4:
  * Graph
    + Split ortools.graph.pywrapgraph into:
  - ortools.graph.python.linear_sum_assignment
  - ortools.graph.python.max_flow
  - ortools.graph.python.min_cost_flow
    This allows using numpy to speed up the setup of the problems.
  * CP-SAT
    + Some improvement on:
  - scheduling (propagation, cuts, lower bounds)
  - MaxSAT (presolve, core based heuristics)
  - MIP performance (presolve, cuts)
- Drop upstream patches:
  * fix_re2_string_view.patch
  * 0001-Build-Python-modules-as-CMake-MODULEs.patch
  * fix_malformed_Cpp.patch
  * 0001-Set-SWIGWORDSIZE-dependent-on-architecture-bitness.patch
- Add patches:
  * do_not_call_pdlp_solve_test.patch
  * 0003-Reuse-common-add_cxx_example-function-for-Sat-Runner.patch
* Fri Jul 15 2022 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Fix failing builds due to stricter RUNPATH checks, add
  * 0001-Only-add-relevant-directories-to-sample-RUNPATHs.patch
  * 0002-Only-add-relevant-directories-to-flatzinc-library-ex.patch
* Sun May 29 2022 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Fix failing build with GCC 12, add fix_malformed_Cpp.patch
- Fix build on 32bit archs, allow tests to fail
- Add _constraints to avoid no-space-left errors
* Sun Apr 24 2022 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Initial package version (9.3)
- Add fix_re2_string_view.patch
- Add 0001-Build-Python-modules-as-CMake-MODULEs.patch
